@@ -20,21 +20,24 @@ try:
 except ImportError:
     from ordereddict import OrderedDict
 import hashlib
+import logging
 import os
 import sys
+import traceback
 import uuid
 
 import fixtures
 from keystoneclient import exceptions as ks_exc
 from keystoneclient import fixture as ks_fixture
 import mock
-import requests
 from requests_mock.contrib import fixture as rm_fixture
 import six
 
 from glanceclient.common import utils
 from glanceclient import exc
 from glanceclient import shell as openstack_shell
+from glanceclient.tests.unit.v2.fixtures import image_show_fixture
+from glanceclient.tests.unit.v2.fixtures import image_versions_fixture
 from glanceclient.tests import utils as testutils
 
 # NOTE (esheffield) Used for the schema caching tests
@@ -42,7 +45,8 @@ from glanceclient.v2 import schemas as schemas
 import json
 
 
-DEFAULT_IMAGE_URL = 'http://127.0.0.1:5000/'
+DEFAULT_IMAGE_URL = 'http://127.0.0.1:9292/'
+DEFAULT_IMAGE_URL_INTERNAL = 'http://127.0.0.1:9191/'
 DEFAULT_USERNAME = 'username'
 DEFAULT_PASSWORD = 'password'
 DEFAULT_TENANT_ID = 'tenant_id'
@@ -54,6 +58,8 @@ DEFAULT_V2_AUTH_URL = '%sv2.0' % DEFAULT_UNVERSIONED_AUTH_URL
 DEFAULT_V3_AUTH_URL = '%sv3' % DEFAULT_UNVERSIONED_AUTH_URL
 DEFAULT_AUTH_TOKEN = ' 3bcc3d3a03f44e3d8377f9247b0ad155'
 TEST_SERVICE_URL = 'http://127.0.0.1:5000/'
+DEFAULT_SERVICE_TYPE = 'image'
+DEFAULT_ENDPOINT_TYPE = 'public'
 
 FAKE_V2_ENV = {'OS_USERNAME': DEFAULT_USERNAME,
                'OS_PASSWORD': DEFAULT_PASSWORD,
@@ -68,6 +74,15 @@ FAKE_V3_ENV = {'OS_USERNAME': DEFAULT_USERNAME,
                'OS_AUTH_URL': DEFAULT_V3_AUTH_URL,
                'OS_IMAGE_URL': DEFAULT_IMAGE_URL}
 
+FAKE_V4_ENV = {'OS_USERNAME': DEFAULT_USERNAME,
+               'OS_PASSWORD': DEFAULT_PASSWORD,
+               'OS_PROJECT_ID': DEFAULT_PROJECT_ID,
+               'OS_USER_DOMAIN_NAME': DEFAULT_USER_DOMAIN_NAME,
+               'OS_AUTH_URL': DEFAULT_V3_AUTH_URL,
+               'OS_SERVICE_TYPE': DEFAULT_SERVICE_TYPE,
+               'OS_ENDPOINT_TYPE': DEFAULT_ENDPOINT_TYPE,
+               'OS_AUTH_TOKEN': DEFAULT_AUTH_TOKEN}
+
 TOKEN_ID = uuid.uuid4().hex
 
 V2_TOKEN = ks_fixture.V2Token(token_id=TOKEN_ID)
@@ -78,7 +93,8 @@ _s.add_endpoint(DEFAULT_IMAGE_URL)
 V3_TOKEN = ks_fixture.V3Token()
 V3_TOKEN.set_project_scope()
 _s = V3_TOKEN.add_service('image', name='glance')
-_s.add_standard_endpoints(public=DEFAULT_IMAGE_URL)
+_s.add_standard_endpoints(public=DEFAULT_IMAGE_URL,
+                          internal=DEFAULT_IMAGE_URL_INTERNAL)
 
 
 class ShellTest(testutils.TestCase):
@@ -100,7 +116,9 @@ class ShellTest(testutils.TestCase):
         self.requests = self.useFixture(rm_fixture.Fixture())
 
         json_list = ks_fixture.DiscoveryList(DEFAULT_UNVERSIONED_AUTH_URL)
-        self.requests.get(DEFAULT_IMAGE_URL, json=json_list, status_code=300)
+        self.requests.get(DEFAULT_UNVERSIONED_AUTH_URL,
+                          json=json_list,
+                          status_code=300)
 
         json_v2 = {'version': ks_fixture.V2Discovery(DEFAULT_V2_AUTH_URL)}
         self.requests.get(DEFAULT_V2_AUTH_URL, json=json_v2)
@@ -150,17 +168,55 @@ class ShellTest(testutils.TestCase):
         argstr = '--os-image-api-version 2 help foofoo'
         self.assertRaises(exc.CommandError, shell.main, argstr.split())
 
+    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stderr', six.StringIO())
+    @mock.patch('sys.argv', ['glance', 'help', 'foofoo'])
+    def test_no_stacktrace_when_debug_disabled(self):
+        with mock.patch.object(traceback, 'print_exc') as mock_print_exc:
+            try:
+                openstack_shell.main()
+            except SystemExit:
+                pass
+            self.assertFalse(mock_print_exc.called)
+
+    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stderr', six.StringIO())
+    @mock.patch('sys.argv', ['glance', 'help', 'foofoo'])
+    def test_stacktrace_when_debug_enabled_by_env(self):
+        old_environment = os.environ.copy()
+        os.environ = {'GLANCECLIENT_DEBUG': '1'}
+        try:
+            with mock.patch.object(traceback, 'print_exc') as mock_print_exc:
+                try:
+                    openstack_shell.main()
+                except SystemExit:
+                    pass
+                self.assertTrue(mock_print_exc.called)
+        finally:
+            os.environ = old_environment
+
+    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stderr', six.StringIO())
+    @mock.patch('sys.argv', ['glance', '--debug', 'help', 'foofoo'])
+    def test_stacktrace_when_debug_enabled(self):
+        with mock.patch.object(traceback, 'print_exc') as mock_print_exc:
+            try:
+                openstack_shell.main()
+            except SystemExit:
+                pass
+            self.assertTrue(mock_print_exc.called)
+
     def test_help(self):
         shell = openstack_shell.OpenStackImagesShell()
         argstr = '--os-image-api-version 2 help'
-        with mock.patch.object(shell, '_get_endpoint_and_token') as et_mock:
+        with mock.patch.object(shell, '_get_keystone_auth_plugin') as et_mock:
             actual = shell.main(argstr.split())
             self.assertEqual(0, actual)
             self.assertFalse(et_mock.called)
 
     def test_blank_call(self):
         shell = openstack_shell.OpenStackImagesShell()
-        with mock.patch.object(shell, '_get_endpoint_and_token') as et_mock:
+        with mock.patch.object(shell, '_get_keystone_auth_plugin') as et_mock:
             actual = shell.main('')
             self.assertEqual(0, actual)
             self.assertFalse(et_mock.called)
@@ -172,7 +228,21 @@ class ShellTest(testutils.TestCase):
     def test_help_v2_no_schema(self):
         shell = openstack_shell.OpenStackImagesShell()
         argstr = '--os-image-api-version 2 help image-create'
-        with mock.patch.object(shell, '_get_endpoint_and_token') as et_mock:
+        with mock.patch.object(shell, '_get_keystone_auth_plugin') as et_mock:
+            actual = shell.main(argstr.split())
+            self.assertEqual(0, actual)
+            self.assertNotIn('<unavailable>', actual)
+            self.assertFalse(et_mock.called)
+
+        argstr = '--os-image-api-version 2 help md-namespace-create'
+        with mock.patch.object(shell, '_get_keystone_auth_plugin') as et_mock:
+            actual = shell.main(argstr.split())
+            self.assertEqual(0, actual)
+            self.assertNotIn('<unavailable>', actual)
+            self.assertFalse(et_mock.called)
+
+        argstr = '--os-image-api-version 2 help md-resource-type-associate'
+        with mock.patch.object(shell, '_get_keystone_auth_plugin') as et_mock:
             actual = shell.main(argstr.split())
             self.assertEqual(0, actual)
             self.assertNotIn('<unavailable>', actual)
@@ -272,18 +342,16 @@ class ShellTest(testutils.TestCase):
         glance_shell = openstack_shell.OpenStackImagesShell()
         glance_shell.main(args.split())
         # NOTE(flaper87): this currently calls auth twice since it'll
-        # authenticate to get the verison list *and* to excuted the command.
+        # authenticate to get the version list *and* to execute the command.
         # This is not the ideal behavior and it should be fixed in a follow
         # up patch.
-        self._assert_auth_plugin_args()
 
     @mock.patch('glanceclient.v1.client.Client')
     def test_auth_plugin_invocation_with_v1(self, v1_client):
         args = '--os-image-api-version 1 image-list'
         glance_shell = openstack_shell.OpenStackImagesShell()
         glance_shell.main(args.split())
-        self.assertEqual(1, self.v2_auth.call_count)
-        self._assert_auth_plugin_args()
+        self.assertEqual(0, self.v2_auth.call_count)
 
     @mock.patch('glanceclient.v2.client.Client')
     def test_auth_plugin_invocation_with_v2(self,
@@ -291,8 +359,7 @@ class ShellTest(testutils.TestCase):
         args = '--os-image-api-version 2 image-list'
         glance_shell = openstack_shell.OpenStackImagesShell()
         glance_shell.main(args.split())
-        self.assertEqual(1, self.v2_auth.call_count)
-        self._assert_auth_plugin_args()
+        self.assertEqual(0, self.v2_auth.call_count)
 
     @mock.patch('glanceclient.v1.client.Client')
     def test_auth_plugin_invocation_with_unversioned_auth_url_with_v1(
@@ -301,7 +368,6 @@ class ShellTest(testutils.TestCase):
                 DEFAULT_UNVERSIONED_AUTH_URL)
         glance_shell = openstack_shell.OpenStackImagesShell()
         glance_shell.main(args.split())
-        self._assert_auth_plugin_args()
 
     @mock.patch('glanceclient.v2.client.Client')
     @mock.patch.object(openstack_shell.OpenStackImagesShell, '_cache_schemas',
@@ -312,7 +378,6 @@ class ShellTest(testutils.TestCase):
                 'image-list') % DEFAULT_UNVERSIONED_AUTH_URL
         glance_shell = openstack_shell.OpenStackImagesShell()
         glance_shell.main(args.split())
-        self._assert_auth_plugin_args()
 
     @mock.patch('glanceclient.Client')
     def test_endpoint_token_no_auth_req(self, mock_client):
@@ -333,23 +398,6 @@ class ShellTest(testutils.TestCase):
         glance_shell.main(args)
         self.assertEqual(1, mock_client.call_count)
 
-    @mock.patch('sys.stdin', side_effect=mock.MagicMock)
-    @mock.patch('getpass.getpass', return_value='password')
-    @mock.patch('glanceclient.v2.client.Client')
-    def test_password_prompted_with_v2(self, v2_client,
-                                       mock_getpass, mock_stdin):
-        self.requests.post(self.token_url, exc=requests.ConnectionError)
-
-        cli2 = mock.MagicMock()
-        v2_client.return_value = cli2
-        cli2.http_client.get.return_value = (None, {'versions': []})
-        glance_shell = openstack_shell.OpenStackImagesShell()
-        self.make_env(exclude='OS_PASSWORD')
-        self.assertRaises(ks_exc.ConnectionRefused,
-                          glance_shell.main, ['image-list'])
-        # Make sure we are actually prompted.
-        mock_getpass.assert_called_once_with('OS Password: ')
-
     @mock.patch('sys.stdin', side_effect=mock.MagicMock)
     @mock.patch('getpass.getpass', side_effect=EOFError)
     @mock.patch('glanceclient.v2.client.Client')
@@ -367,7 +415,7 @@ class ShellTest(testutils.TestCase):
         mock_getpass.assert_called_with('OS Password: ')
 
     @mock.patch(
-        'glanceclient.shell.OpenStackImagesShell._get_keystone_session')
+        'glanceclient.shell.OpenStackImagesShell._get_keystone_auth_plugin')
     @mock.patch.object(openstack_shell.OpenStackImagesShell, '_cache_schemas',
                        return_value=False)
     def test_no_auth_with_proj_name(self, cache_schemas, session):
@@ -533,9 +581,28 @@ class ShellTest(testutils.TestCase):
         except SystemExit:
             self.fail('Unexpected SystemExit')
 
-        # We expect the normal usage as a result
-        self.assertIn('Command-line interface to the OpenStack Images API',
-                      sys.stdout.getvalue())
+        # We expect the normal v2 usage as a result
+        expected = ['Command-line interface to the OpenStack Images API',
+                    'image-list',
+                    'image-deactivate',
+                    'location-add']
+        for output in expected:
+            self.assertIn(output,
+                          sys.stdout.getvalue())
+
+    @mock.patch('glanceclient.v2.client.Client')
+    @mock.patch('glanceclient.v1.shell.do_image_list')
+    @mock.patch('glanceclient.shell.logging.basicConfig')
+    def test_setup_debug(self, conf, func, v2_client):
+        cli2 = mock.MagicMock()
+        v2_client.return_value = cli2
+        cli2.http_client.get.return_value = (None, {'versions': []})
+        args = '--debug image-list'
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        glance_logger = logging.getLogger('glanceclient')
+        self.assertEqual(glance_logger.getEffectiveLevel(), logging.DEBUG)
+        conf.assert_called_with(level=logging.DEBUG)
 
 
 class ShellTestWithKeystoneV3Auth(ShellTest):
@@ -561,16 +628,14 @@ class ShellTestWithKeystoneV3Auth(ShellTest):
         args = '--os-image-api-version 1 image-list'
         glance_shell = openstack_shell.OpenStackImagesShell()
         glance_shell.main(args.split())
-        self.assertEqual(1, self.v3_auth.call_count)
-        self._assert_auth_plugin_args()
+        self.assertEqual(0, self.v3_auth.call_count)
 
     @mock.patch('glanceclient.v2.client.Client')
     def test_auth_plugin_invocation_with_v2(self, v2_client):
         args = '--os-image-api-version 2 image-list'
         glance_shell = openstack_shell.OpenStackImagesShell()
         glance_shell.main(args.split())
-        self.assertEqual(1, self.v3_auth.call_count)
-        self._assert_auth_plugin_args()
+        self.assertEqual(0, self.v3_auth.call_count)
 
     @mock.patch('keystoneclient.discover.Discover',
                 side_effect=ks_exc.ClientException())
@@ -598,6 +663,65 @@ class ShellTestWithKeystoneV3Auth(ShellTest):
             self.assertNotIn(r, stdout.split())
 
 
+class ShellTestWithNoOSImageURLPublic(ShellTestWithKeystoneV3Auth):
+    # auth environment to use
+    # default uses public
+    auth_env = FAKE_V4_ENV.copy()
+
+    def setUp(self):
+        super(ShellTestWithNoOSImageURLPublic, self).setUp()
+        self.image_url = DEFAULT_IMAGE_URL
+        self.requests.get(DEFAULT_IMAGE_URL + 'v2/images',
+                          text='{"images": []}')
+
+    @mock.patch('glanceclient.v1.client.Client')
+    def test_auth_plugin_invocation_with_v1(self, v1_client):
+        args = '--os-image-api-version 1 image-list'
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        self.assertEqual(1, self.v3_auth.call_count)
+        self._assert_auth_plugin_args()
+
+    @mock.patch('glanceclient.v2.client.Client')
+    def test_auth_plugin_invocation_with_v2(self, v2_client):
+        args = '--os-image-api-version 2 image-list'
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        self.assertEqual(1, self.v3_auth.call_count)
+        self._assert_auth_plugin_args()
+
+    @mock.patch('glanceclient.v2.client.Client')
+    def test_endpoint_from_interface(self, v2_client):
+        args = ('--os-image-api-version 2 image-list')
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        assert v2_client.called
+        (args, kwargs) = v2_client.call_args
+        self.assertEqual(self.image_url, kwargs['endpoint_override'])
+
+    def test_endpoint_real_from_interface(self):
+        args = ('--os-image-api-version 2 image-list')
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        self.assertEqual(self.requests.request_history[2].url,
+                         self.image_url + "v2/images?limit=20&"
+                         "sort_key=name&sort_dir=asc")
+
+
+class ShellTestWithNoOSImageURLInternal(ShellTestWithNoOSImageURLPublic):
+    # auth environment to use
+    # this uses internal
+    FAKE_V5_ENV = FAKE_V4_ENV.copy()
+    FAKE_V5_ENV['OS_ENDPOINT_TYPE'] = 'internal'
+    auth_env = FAKE_V5_ENV.copy()
+
+    def setUp(self):
+        super(ShellTestWithNoOSImageURLPublic, self).setUp()
+        self.image_url = DEFAULT_IMAGE_URL_INTERNAL
+        self.requests.get(DEFAULT_IMAGE_URL_INTERNAL + 'v2/images',
+                          text='{"images": []}')
+
+
 class ShellCacheSchemaTest(testutils.TestCase):
     def setUp(self):
         super(ShellCacheSchemaTest, self).setUp()
@@ -627,9 +751,11 @@ class ShellCacheSchemaTest(testutils.TestCase):
         self.client.schemas.get.return_value = schemas.Schema(schema_odict)
 
     def _mock_shell_setup(self):
-        mocked_get_client = mock.MagicMock(return_value=self.client)
         self.shell = openstack_shell.OpenStackImagesShell()
-        self.shell._get_versioned_client = mocked_get_client
+        self.shell._get_versioned_client = mock.create_autospec(
+            self.shell._get_versioned_client, return_value=self.client,
+            spec_set=True
+        )
 
     def _make_args(self, args):
         class Args(object):
@@ -648,7 +774,7 @@ class ShellCacheSchemaTest(testutils.TestCase):
         schema_odict = OrderedDict(self.schema_dict)
 
         args = self._make_args(options)
-        client = self.shell._get_versioned_client('2', args, force_auth=True)
+        client = self.shell._get_versioned_client('2', args)
         self.shell._cache_schemas(args, client, home_dir=self.cache_dir)
 
         self.assertEqual(12, open.mock_calls.__len__())
@@ -671,7 +797,7 @@ class ShellCacheSchemaTest(testutils.TestCase):
         schema_odict = OrderedDict(self.schema_dict)
 
         args = self._make_args(options)
-        client = self.shell._get_versioned_client('2', args, force_auth=True)
+        client = self.shell._get_versioned_client('2', args)
         self.shell._cache_schemas(args, client, home_dir=self.cache_dir)
 
         self.assertEqual(12, open.mock_calls.__len__())
@@ -696,9 +822,12 @@ class ShellCacheSchemaTest(testutils.TestCase):
         self.shell._cache_schemas(self._make_args(options),
                                   client, home_dir=self.cache_dir)
 
-        os.path.exists.assert_any_call(self.prefix_path)
-        os.path.exists.assert_any_call(self.cache_files[0])
-        os.path.exists.assert_any_call(self.cache_files[1])
+        exists_mock.assert_has_calls([
+            mock.call(self.prefix_path),
+            mock.call(self.cache_files[0]),
+            mock.call(self.cache_files[1]),
+            mock.call(self.cache_files[2])
+        ])
         self.assertEqual(4, exists_mock.call_count)
         self.assertEqual(0, open.mock_calls.__len__())
 
@@ -716,4 +845,116 @@ class ShellCacheSchemaTest(testutils.TestCase):
         switch_version = self.shell._cache_schemas(self._make_args(options),
                                                    client,
                                                    home_dir=self.cache_dir)
-        self.assertEqual(switch_version, True)
+        self.assertEqual(True, switch_version)
+
+
+class ShellTestRequests(testutils.TestCase):
+    """Shell tests using the requests mock library."""
+    def _make_args(self, args):
+        # NOTE(venkatesh): this conversion from a dict to an object
+        # is required because the test_shell.do_xxx(gc, args) methods
+        # expects the args to be attributes of an object. If passed as
+        # dict directly, it throws an AttributeError.
+        class Args(object):
+            def __init__(self, entries):
+                self.__dict__.update(entries)
+
+        return Args(args)
+
+    def setUp(self):
+        super(ShellTestRequests, self).setUp()
+        self._old_env = os.environ
+        os.environ = {}
+
+    def tearDown(self):
+        super(ShellTestRequests, self).tearDown()
+        os.environ = self._old_env
+
+    def test_download_has_no_stray_output_to_stdout(self):
+        """Regression test for bug 1488914"""
+        saved_stdout = sys.stdout
+        try:
+            sys.stdout = output = testutils.FakeNoTTYStdout()
+            id = image_show_fixture['id']
+            self.requests = self.useFixture(rm_fixture.Fixture())
+            self.requests.get('http://example.com/versions',
+                              json=image_versions_fixture)
+
+            headers = {'Content-Length': '4',
+                       'Content-type': 'application/octet-stream'}
+            fake = testutils.FakeResponse(headers, six.StringIO('DATA'))
+            self.requests.get('http://example.com/v1/images/%s' % id,
+                              raw=fake)
+
+            self.requests.get('http://example.com/v1/images/detail'
+                              '?sort_key=name&sort_dir=asc&limit=20')
+
+            headers = {'X-Image-Meta-Id': id}
+            self.requests.head('http://example.com/v1/images/%s' % id,
+                               headers=headers)
+
+            with mock.patch.object(openstack_shell.OpenStackImagesShell,
+                                   '_cache_schemas') as mocked_cache_schema:
+                mocked_cache_schema.return_value = True
+                shell = openstack_shell.OpenStackImagesShell()
+                argstr = ('--os-auth-token faketoken '
+                          '--os-image-url http://example.com '
+                          'image-download %s' % id)
+                shell.main(argstr.split())
+                self.assertTrue(mocked_cache_schema.called)
+            # Ensure we have *only* image data
+            self.assertEqual('DATA', output.getvalue())
+        finally:
+            sys.stdout = saved_stdout
+
+    def test_v1_download_has_no_stray_output_to_stdout(self):
+        """Ensure no stray print statements corrupt the image"""
+        saved_stdout = sys.stdout
+        try:
+            sys.stdout = output = testutils.FakeNoTTYStdout()
+            id = image_show_fixture['id']
+
+            self.requests = self.useFixture(rm_fixture.Fixture())
+            headers = {'X-Image-Meta-Id': id}
+            self.requests.head('http://example.com/v1/images/%s' % id,
+                               headers=headers)
+
+            headers = {'Content-Length': '4',
+                       'Content-type': 'application/octet-stream'}
+            fake = testutils.FakeResponse(headers, six.StringIO('DATA'))
+            self.requests.get('http://example.com/v1/images/%s' % id,
+                              headers=headers, raw=fake)
+
+            shell = openstack_shell.OpenStackImagesShell()
+            argstr = ('--os-image-api-version 1 --os-auth-token faketoken '
+                      '--os-image-url http://example.com '
+                      'image-download %s' % id)
+            shell.main(argstr.split())
+            # Ensure we have *only* image data
+            self.assertEqual('DATA', output.getvalue())
+        finally:
+            sys.stdout = saved_stdout
+
+    def test_v2_download_has_no_stray_output_to_stdout(self):
+        """Ensure no stray print statements corrupt the image"""
+        saved_stdout = sys.stdout
+        try:
+            sys.stdout = output = testutils.FakeNoTTYStdout()
+            id = image_show_fixture['id']
+            headers = {'Content-Length': '4',
+                       'Content-type': 'application/octet-stream'}
+            fake = testutils.FakeResponse(headers, six.StringIO('DATA'))
+
+            self.requests = self.useFixture(rm_fixture.Fixture())
+            self.requests.get('http://example.com/v2/images/%s/file' % id,
+                              headers=headers, raw=fake)
+
+            shell = openstack_shell.OpenStackImagesShell()
+            argstr = ('--os-image-api-version 2 --os-auth-token faketoken '
+                      '--os-image-url http://example.com '
+                      'image-download %s' % id)
+            shell.main(argstr.split())
+            # Ensure we have *only* image data
+            self.assertEqual('DATA', output.getvalue())
+        finally:
+            sys.stdout = saved_stdout
